import gzip
from binascii import hexlify

import requests

from steam import __version__
from steam.core.crypto import sha1_hash, random_bytes


def make_requests_session():
    """
    :returns: requests session
    :rtype: :class:`requests.Session`
    """
    session = requests.Session()
    session.headers['User-Agent'] = f"python-steam/{__version__} {session.headers['User-Agent']}"
    session.hooks["response"].append(decode_gzip)

    return session


def decode_gzip(response, *args, **kwargs):
    # Ensure urllib3 decoding is enabled
    try:
        response.raw.decode_content = True
    except Exception:
        pass

    # Fallback: manually decompress if still gzipped
    raw = response.content
    if raw.startswith(b"\x1f\x8b"):
        try:
            decompressed = gzip.decompress(raw)
            response._content = decompressed  # override safely
            response.encoding = response.encoding or "utf-8"
        except Exception:
            pass

    return response


def generate_session_id():
    """
    :returns: session id
    :rtype: :class:`str`
    """
    return hexlify(sha1_hash(random_bytes(32)))[:32].decode('ascii')



from steam.enums.base import SteamIntEnum


class E_STAR_GlyphWriteResult(SteamIntEnum):
    Success = 0
    InvalidMessage = 1
    InvalidJSON = 2
    SQLError = 3


class EACState(SteamIntEnum):
    Unknown = 0
    Disconnected = 1
    Connected = 2
    ConnectedSlow = 3


class EAgreementType(SteamIntEnum):
    Invalid = -1
    GlobalSSA = 0
    ChinaSSA = 1


class EAudioFormat(SteamIntEnum):
    EAudioFormatNone = 0
    EAudioFormat16BitLittleEndian = 1
    EAudioFormatFloat = 2


EAuthSessionGuardType = SteamIntEnum('EAuthSessionGuardType', {
    'Unknown': 0,
    'None': 1,
    'EmailCode': 2,
    'DeviceCode': 3,
    'DeviceConfirmation': 4,
    'EmailConfirmation': 5,
    'MachineToken': 6,
    'LegacyMachineAuth': 7
})


class EAuthSessionSecurityHistory(SteamIntEnum):
    Invalid = 0
    UsedPreviously = 1
    NoPriorHistory = 2


class EAuthTokenPlatformType(SteamIntEnum):
    Unknown = 0
    SteamClient = 1
    WebBrowser = 2
    MobileApp = 3


class EAuthTokenRevokeAction(SteamIntEnum):
    EAuthTokenRevokeLogout = 0
    EAuthTokenRevokePermanent = 1
    EAuthTokenRevokeReplaced = 2
    EAuthTokenRevokeSupport = 3


class EAuthTokenState(SteamIntEnum):
    Invalid = 0
    New = 1
    Confirmed = 2
    Issued = 3
    Denied = 4
    LoggedOut = 5
    Revoked = 99


class EBatteryState(SteamIntEnum):
    Unknown = 0
    Discharging = 1
    Charging = 2
    Full = 3


class EBluetoothDeviceType(SteamIntEnum):
    BluetoothDeviceType_Invalid = 0
    BluetoothDeviceType_Unknown = 1
    BluetoothDeviceType_Phone = 2
    BluetoothDeviceType_Computer = 3
    BluetoothDeviceType_Headset = 4
    BluetoothDeviceType_Headphones = 5
    BluetoothDeviceType_Speakers = 6
    BluetoothDeviceType_OtherAudio = 7
    BluetoothDeviceType_Mouse = 8
    BluetoothDeviceType_Joystick = 9
    BluetoothDeviceType_Gamepad = 10
    BluetoothDeviceType_Keyboard = 11


class EBroadcastChatPermission(SteamIntEnum):
    EBroadcastChatPermissionPublic = 0
    EBroadcastChatPermissionOwnsApp = 1


EBroadcastImageType = SteamIntEnum('EBroadcastImageType', {
    'None': 0,
    'Offline': 1,
    'Standby': 2,
    'Avatar': 3,
    'Summary': 4,
    'Background': 5,
    'Emoticon': 6,
})


class EBroadcastWatchLocation(SteamIntEnum):
    Invalid = 0
    SteamTV_Tab = 1
    SteamTV_WatchParty = 2
    Chat_Tab = 3
    Chat_WatchParty = 4
    CommunityPage = 5
    StoreAppPage = 6
    InGame = 7
    BigPicture = 8
    SalesPage = 9
    CuratorPage = 10
    DeveloperPage = 11
    Chat_Friends = 12
    SteamTV_Web = 13


class EChatRoomGroupRank(SteamIntEnum):
    Default = 0
    Viewer = 10
    Guest = 15
    Member = 20
    Moderator = 30
    Officer = 40
    Owner = 50
    TestInvalid = 99


EChatRoomJoinState = SteamIntEnum('EChatRoomJoinState', {
    'Default': 0,
    'None': 1,
    'Joined': 2,
    'TestInvalid': 99,
})


class EChatRoomMemberStateChange(SteamIntEnum):
    Invalid = 0
    Joined = 1
    Parted = 2
    Kicked = 3
    Invited = 4
    RankChanged = 7
    InviteDismissed = 8
    Muted = 9
    Banned = 10
    RolesChanged = 12


class EChatRoomMessageReactionType(SteamIntEnum):
    Invalid = 0
    Emoticon = 1
    Sticker = 2


class EChatRoomNotificationLevel(SteamIntEnum):
    EChatroomNotificationLevel_Invalid = 0
    EChatroomNotificationLevel_None = 1
    EChatroomNotificationLevel_MentionMe = 2
    EChatroomNotificationLevel_MentionAll = 3
    EChatroomNotificationLevel_AllMessages = 4


class EChatRoomServerMessage(SteamIntEnum):
    EChatRoomServerMsg_Invalid = 0
    EChatRoomServerMsg_RenameChatRoom = 1
    EChatRoomServerMsg_Joined = 2
    EChatRoomServerMsg_Parted = 3
    EChatRoomServerMsg_Kicked = 4
    EChatRoomServerMsg_Invited = 5
    EChatRoomServerMsg_InviteDismissed = 8
    EChatRoomServerMsg_ChatRoomTaglineChanged = 9
    EChatRoomServerMsg_ChatRoomAvatarChanged = 10
    EChatRoomServerMsg_AppCustom = 11


class ECloudStoragePersistState(SteamIntEnum):
    ECloudStoragePersistStatePersisted = 0
    ECloudStoragePersistStateForgotten = 1
    ECloudStoragePersistStateDeleted = 2


class EColorProfile(SteamIntEnum):
    Invalid = 0
    Native = 1
    Standard = 2
    Vivid = 3


class ECommunityItemClass(SteamIntEnum):
    Invalid = 0
    Badge = 1
    GameCard = 2
    ProfileBackground = 3
    Emoticon = 4
    BoosterPack = 5
    Consumable = 6
    GameGoo = 7
    ProfileModifier = 8
    Scene = 9
    SalienItem = 10
    Sticker = 11
    ChatEffect = 12
    MiniProfileBackground = 13
    AvatarFrame = 14
    AnimatedAvatar = 15
    SteamDeckKeyboardSkin = 16


class EContentCheckProvider(SteamIntEnum):
    Invalid = 0
    Google = 1
    Amazon = 2
    Local = 3


class ECPUGovernor(SteamIntEnum):
    Invalid = 0
    Perf = 1
    Powersave = 2
    Manual = 3


class EExternalAccountType(SteamIntEnum):
    EExternalNone = 0
    EExternalSteamAccount = 1
    EExternalGoogleAccount = 2
    EExternalFacebookAccount = 3
    EExternalTwitterAccount = 4
    EExternalTwitchAccount = 5
    EExternalYouTubeChannelAccount = 6
    EExternalFacebookPage = 7


class EFrameAccumulatedStat(SteamIntEnum):
    EFrameStatFPS = 0
    EFrameStatCaptureDurationMS = 1
    EFrameStatConvertDurationMS = 2
    EFrameStatEncodeDurationMS = 3
    EFrameStatSteamDurationMS = 4
    EFrameStatServerDurationMS = 5
    EFrameStatNetworkDurationMS = 6
    EFrameStatDecodeDurationMS = 7
    EFrameStatDisplayDurationMS = 8
    EFrameStatClientDurationMS = 9
    EFrameStatFrameDurationMS = 10
    EFrameStatInputLatencyMS = 11
    EFrameStatGameLatencyMS = 12
    EFrameStatRoundTripLatencyMS = 13
    EFrameStatPingTimeMS = 14
    EFrameStatServerBitrateKbitPerSec = 15
    EFrameStatClientBitrateKbitPerSec = 16
    EFrameStatLinkBandwidthKbitPerSec = 17
    EFrameStatPacketLossPercentage = 18


EGameSearchAction = SteamIntEnum('EGameSearchAction', {
    'None': 0,
    'Accept': 1,
    'Decline': 2,
    'Cancel': 3,
})


class EGameSearchResult(SteamIntEnum):
    Invalid = 0
    SearchInProgress = 1
    SearchFailedNoHosts = 2
    SearchGameFound = 3
    SearchCompleteAccepted = 4
    SearchCompleteDeclined = 5
    SearchCanceled = 6


class EGetChannelsAlgorithm(SteamIntEnum):
    Default = 1
    Friends = 2
    Featured = 3
    Developer = 4
    Following = 5


class EGetGamesAlgorithm(SteamIntEnum):
    Default = 1
    MostPlayed = 2
    PopularNew = 3


class EGPUPerformanceLevel(SteamIntEnum):
    Invalid = 0
    Auto = 1
    Manual = 2
    Low = 3
    High = 4
    Profiling = 5


class EGraphicsPerfOverlayLevel(SteamIntEnum):
    Hidden = 0
    Basic = 1
    Medium = 2
    Full = 3
    Minimal = 4


class EInternalAccountType(SteamIntEnum):
    EInternalSteamAccountType = 1
    EInternalClanType = 2
    EInternalAppType = 3
    EInternalBroadcastChannelType = 4


class EKeyEscrowUsage(SteamIntEnum):
    EKeyEscrowUsageStreamingDevice = 0


class ELobbyStatus(SteamIntEnum):
    ELobbyStatusInvalid = 0
    ELobbyStatusExists = 1
    ELobbyStatusDoesNotExist = 2
    ELobbyStatusNotAMember = 3


class ELogFileType(SteamIntEnum):
    ELogFileSystemBoot = 0
    ELogFileSystemReset = 1
    ELogFileSystemDebug = 2


class EMarketingMessageAssociationType(SteamIntEnum):
    EMarketingMessageNoAssociation = 0
    EMarketingMessageAppAssociation = 1
    EMarketingMessageSubscriptionAssociation = 2
    EMarketingMessagePublisherAssociation = 3
    EMarketingMessageGenreAssociation = 4
    EMarketingMessageBundleAssociation = 5


class EMarketingMessageLookupType(SteamIntEnum):
    EMarketingMessageLookupInvalid = 0
    EMarketingMessageLookupByGID = 1
    EMarketingMessageLookupActive = 2
    EMarketingMessageLookupByTitleWithType = 3
    EMarketingMessageLookupByGIDList = 4


class EMarketingMessageType(SteamIntEnum):
    EMarketingMessageInvalid = 0
    EMarketingMessageNowAvailable = 1
    EMarketingMessageWeekendDeal = 2
    EMarketingMessagePrePurchase = 3
    EMarketingMessagePlayNow = 4
    EMarketingMessagePreloadNow = 5
    EMarketingMessageGeneral = 6
    EMarketingMessageDemoQuit = 7
    EMarketingMessageGifting = 8
    EMarketingMessageEJsKorner = 9


class EMarketingMessageVisibility(SteamIntEnum):
    EMarketingMessageVisibleBeta = 1
    EMarketingMessageVisiblePublic = 2


class EMessageReactionType(SteamIntEnum):
    Invalid = 0
    Emoticon = 1
    Sticker = 2


class ENotificationSetting(SteamIntEnum):
    ENotificationSettingNotifyUseDefault = 0
    ENotificationSettingAlways = 1
    ENotificationSettingNever = 2


class EOSBranch(SteamIntEnum):
    Unknown = 0
    Release = 1
    ReleaseCandidate = 2
    Beta = 3
    BetaCandidate = 4
    Main = 5


class EPlaytestStatus(SteamIntEnum):
    ETesterStatusNone = 0
    ETesterStatusPending = 1
    ETesterStatusInvited = 2
    ETesterStatusGranted = 3


class EProfileCustomizationStyle(SteamIntEnum):
    EProfileCustomizationStyleDefault = 0
    EProfileCustomizationStyleSelected = 1
    EProfileCustomizationStyleRarest = 2
    EProfileCustomizationStyleMostRecent = 3
    EProfileCustomizationStyleRandom = 4
    EProfileCustomizationStyleHighestRated = 5


class EProfileCustomizationType(SteamIntEnum):
    EProfileCustomizationTypeInvalid = 0
    EProfileCustomizationTypeRareAchievementShowcase = 1
    EProfileCustomizationTypeGameCollector = 2
    EProfileCustomizationTypeItemShowcase = 3
    EProfileCustomizationTypeTradeShowcase = 4
    EProfileCustomizationTypeBadges = 5
    EProfileCustomizationTypeFavoriteGame = 6
    EProfileCustomizationTypeScreenshotShowcase = 7
    EProfileCustomizationTypeCustomText = 8
    EProfileCustomizationTypeFavoriteGroup = 9
    EProfileCustomizationTypeRecommendation = 10
    EProfileCustomizationTypeWorkshopItem = 11
    EProfileCustomizationTypeMyWorkshop = 12
    EProfileCustomizationTypeArtworkShowcase = 13
    EProfileCustomizationTypeVideoShowcase = 14
    EProfileCustomizationTypeGuides = 15
    EProfileCustomizationTypeMyGuides = 16
    EProfileCustomizationTypeAchievements = 17
    EProfileCustomizationTypeGreenlight = 18
    EProfileCustomizationTypeMyGreenlight = 19
    EProfileCustomizationTypeSalien = 20
    EProfileCustomizationTypeLoyaltyRewardReactions = 21
    EProfileCustomizationTypeSingleArtworkShowcase = 22
    EProfileCustomizationTypeAchievementsCompletionist = 23


class EProtoExecutionSite(SteamIntEnum):
    EProtoExecutionSiteUnknown = 0
    EProtoExecutionSiteSteamClient = 2


class EProvideDeckFeedbackPreference(SteamIntEnum):
    Unset = 0
    Yes = 1
    No = 2


class EPublishedFileForSaleStatus(SteamIntEnum):
    PFFSS_NotForSale = 0
    PFFSS_PendingApproval = 1
    PFFSS_ApprovedForSale = 2
    PFFSS_RejectedForSale = 3
    PFFSS_NoLongerForSale = 4
    PFFSS_TentativeApproval = 5


class EPublishedFileRevision(SteamIntEnum):
    Default = 0
    Latest = 1
    ApprovedSnapshot = 2
    ApprovedSnapshot_China = 3
    RejectedSnapshot = 4
    RejectedSnapshot_China = 5


class EPublishedFileStorageSystem(SteamIntEnum):
    EPublishedFileStorageSystemInvalid = 0
    EPublishedFileStorageSystemLegacyCloud = 1
    EPublishedFileStorageSystemDepot = 2
    EPublishedFileStorageSystemUGCCloud = 3


class EScalingFilter(SteamIntEnum):
    Invalid = 0
    FSR = 1
    Nearest = 2
    Integer = 3
    Linear = 4
    NIS = 5


class ESDCardFormatStage(SteamIntEnum):
    Invalid = 0
    Starting = 1
    Testing = 2
    Rescuing = 3
    Formatting = 4
    Finalizing = 5


class ESessionPersistence(SteamIntEnum):
    Invalid = -1
    Ephemeral = 0
    Persistent = 1


class ESteamDeckCompatibilityCategory(SteamIntEnum):
    Unknown = 0
    Unsupported = 1
    Playable = 2
    Verified = 3


class ESteamDeckCompatibilityFeedback(SteamIntEnum):
    Unset = 0
    Agree = 1
    Disagree = 2
    Ignore = 3


class ESteamDeckCompatibilityResultDisplayType(SteamIntEnum):
    Invisible = 0
    Informational = 1
    Unsupported = 2
    Playable = 3
    Verified = 4


class ESteamTVContentTemplate(SteamIntEnum):
    Invalid = 0
    Takeover = 1
    SingleGame = 2
    GameList = 3
    QuickExplore = 4
    ConveyorBelt = 5
    WatchParty = 6
    Developer = 7
    Event = 8


class EStorageBlockContentType(SteamIntEnum):
    Invalid = 0
    Unknown = 1
    FileSystem = 2
    Crypto = 3
    Raid = 4


class EStorageBlockFileSystemType(SteamIntEnum):
    Invalid = 0
    Unknown = 1
    VFat = 2
    Ext4 = 3


class EStoreAppType(SteamIntEnum):
    Game = 0
    Demo = 1
    Mod = 2
    Movie = 3
    DLC = 4
    Guide = 5
    Software = 6
    Video = 7
    Series = 8
    Episode = 9
    Hardware = 10
    Music = 11
    Beta = 12
    Tool = 13
    Advertising = 14


class EStoreCategoryType(SteamIntEnum):
    Category = 0
    SupportedPlayers = 1
    Feature = 2
    ControllerSupport = 3
    CloudGaming = 4
    MAX = 5


class EStoreDiscoveryQueueType(SteamIntEnum):
    EStoreDiscoveryQueueTypeNew = 0
    EStoreDiscoveryQueueTypeComingSoon = 1
    EStoreDiscoveryQueueTypeRecommended = 2
    EStoreDiscoveryQueueTypeEveryNewRelease = 3
    EStoreDiscoveryQueueTypeMLRecommender = 5
    EStoreDiscoveryQueueTypeWishlistOnSale = 6
    EStoreDiscoveryQueueTypeDLC = 7
    EStoreDiscoveryQueueTypeDLCOnSale = 8
    EStoreDiscoveryQueueTypeRecommendedComingSoon = 9
    EStoreDiscoveryQueueTypeRecommendedFree = 10
    EStoreDiscoveryQueueTypeRecommendedOnSale = 11
    EStoreDiscoveryQueueTypeRecommendedDemos = 12
    EStoreDiscoveryQueueTypeDLCNewReleases = 13
    EStoreDiscoveryQueueTypeDLCTopSellers = 14
    EStoreDiscoveryQueueTypeMAX = 15


class EStoreItemType(SteamIntEnum):
    Invalid = -1
    App = 0
    Package = 1
    Bundle = 2
    Mtx = 3


class EStreamActivity(SteamIntEnum):
    EStreamActivityIdle = 1
    EStreamActivityGame = 2
    EStreamActivityDesktop = 3
    EStreamActivitySecureDesktop = 4
    EStreamActivityMusic = 5


class EStreamAudioCodec(SteamIntEnum):
    EStreamAudioCodecNone = 0
    EStreamAudioCodecRaw = 1
    EStreamAudioCodecVorbis = 2
    EStreamAudioCodecOpus = 3
    EStreamAudioCodecMP3 = 4
    EStreamAudioCodecAAC = 5


class EStreamBitrate(SteamIntEnum):
    EStreamBitrateAutodetect = -1
    EStreamBitrateUnlimited = 0


class EStreamChannel(SteamIntEnum):
    EStreamChannelInvalid = -1
    EStreamChannelDiscovery = 0
    EStreamChannelControl = 1
    EStreamChannelStats = 2
    EStreamChannelDataChannelStart = 3


class EStreamColorspace(SteamIntEnum):
    Unknown = 0
    BT601 = 1
    BT601_Full = 2
    BT709 = 3
    BT709_Full = 4


class EStreamControllerConfigMsg(SteamIntEnum):
    RequestConfigsForApp = 0
    ConfigResponse = 1
    PersonalizationResponse = 2
    ActiveConfigChange = 3
    RequestActiveConfig = 4


class EStreamControlMessage(SteamIntEnum):
    EStreamControlAuthenticationRequest = 1
    EStreamControlAuthenticationResponse = 2
    EStreamControlNegotiationInit = 3
    EStreamControlNegotiationSetConfig = 4
    EStreamControlNegotiationComplete = 5
    EStreamControlClientHandshake = 6
    EStreamControlServerHandshake = 7
    EStreamControlStartNetworkTest = 8
    EStreamControlKeepAlive = 9
    EStreamControl_LAST_SETUP_MESSAGE = 15
    EStreamControlStartAudioData = 50
    EStreamControlStopAudioData = 51
    EStreamControlStartVideoData = 52
    EStreamControlStopVideoData = 53
    EStreamControlInputMouseMotion = 54
    EStreamControlInputMouseWheel = 55
    EStreamControlInputMouseDown = 56
    EStreamControlInputMouseUp = 57
    EStreamControlInputKeyDown = 58
    EStreamControlInputKeyUp = 59
    EStreamControlInputGamepadAttached_OBSOLETE = 60
    EStreamControlInputGamepadEvent_OBSOLETE = 61
    EStreamControlInputGamepadDetached_OBSOLETE = 62
    EStreamControlShowCursor = 63
    EStreamControlHideCursor = 64
    EStreamControlSetCursor = 65
    EStreamControlGetCursorImage = 66
    EStreamControlSetCursorImage = 67
    EStreamControlDeleteCursor = 68
    EStreamControlSetTargetFramerate = 69
    EStreamControlInputLatencyTest = 70
    EStreamControlGamepadRumble_OBSOLETE = 71
    EStreamControlOverlayEnabled = 74
    EStreamControlInputControllerAttached_OBSOLETE = 75
    EStreamControlInputControllerState_OBSOLETE = 76
    EStreamControlTriggerHapticPulse_OBSOLETE = 77
    EStreamControlInputControllerDetached_OBSOLETE = 78
    EStreamControlVideoDecoderInfo = 80
    EStreamControlSetTitle = 81
    EStreamControlSetIcon = 82
    EStreamControlQuitRequest = 83
    EStreamControlSetQoS = 87
    EStreamControlInputControllerWirelessPresence_OBSOLETE = 88
    EStreamControlSetGammaRamp = 89
    EStreamControlVideoEncoderInfo = 90
    EStreamControlInputControllerStateHID_OBSOLETE = 93
    EStreamControlSetTargetBitrate = 94
    EStreamControlSetControllerPairingEnabled_OBSOLETE = 95
    EStreamControlSetControllerPairingResult_OBSOLETE = 96
    EStreamControlTriggerControllerDisconnect_OBSOLETE = 97
    EStreamControlSetActivity = 98
    EStreamControlSetStreamingClientConfig = 99
    EStreamControlSystemSuspend = 100
    EStreamControlSetControllerSettings_OBSOLETE = 101
    EStreamControlVirtualHereRequest = 102
    EStreamControlVirtualHereReady = 103
    EStreamControlVirtualHereShareDevice = 104
    EStreamControlSetSpectatorMode = 105
    EStreamControlRemoteHID = 106
    EStreamControlStartMicrophoneData = 107
    EStreamControlStopMicrophoneData = 108
    EStreamControlInputText = 109
    EStreamControlTouchConfigActive = 110
    EStreamControlGetTouchConfigData = 111
    EStreamControlSetTouchConfigData = 112
    EStreamControlSaveTouchConfigLayout = 113
    EStreamControlTouchActionSetActive = 114
    EStreamControlGetTouchIconData = 115
    EStreamControlSetTouchIconData = 116
    EStreamControlInputTouchFingerDown = 117
    EStreamControlInputTouchFingerMotion = 118
    EStreamControlInputTouchFingerUp = 119
    EStreamControlSetCaptureSize = 120
    EStreamControlSetFlashState = 121
    EStreamControlPause = 122
    EStreamControlResume = 123
    EStreamControlEnableHighResCapture = 124
    EStreamControlDisableHighResCapture = 125
    EStreamControlToggleMagnification = 126
    EStreamControlSetCapslock = 127
    EStreamControlSetKeymap = 128
    EStreamControlStopRequest = 129
    EStreamControlTouchActionSetLayerAdded = 130
    EStreamControlTouchActionSetLayerRemoved = 131
    EStreamControlRemotePlayTogetherGroupUpdate = 132
    EStreamControlSetInputTemporarilyDisabled = 133
    EStreamControlSetQualityOverride = 134
    EStreamControlSetBitrateOverride = 135
    EStreamControlShowOnScreenKeyboard = 136
    EStreamControlControllerConfigMsg = 137


class EStreamDataMessage(SteamIntEnum):
    EStreamDataPacket = 1
    EStreamDataLost = 2


class EStreamDiscoveryMessage(SteamIntEnum):
    EStreamDiscoveryPingRequest = 1
    EStreamDiscoveryPingResponse = 2


class EStreamFrameEvent(SteamIntEnum):
    EStreamInputEventStart = 0
    EStreamInputEventSend = 1
    EStreamInputEventRecv = 2
    EStreamInputEventQueued = 3
    EStreamInputEventHandled = 4
    EStreamFrameEventStart = 5
    EStreamFrameEventCaptureBegin = 6
    EStreamFrameEventCaptureEnd = 7
    EStreamFrameEventConvertBegin = 8
    EStreamFrameEventConvertEnd = 9
    EStreamFrameEventEncodeBegin = 10
    EStreamFrameEventEncodeEnd = 11
    EStreamFrameEventSend = 12
    EStreamFrameEventRecv = 13
    EStreamFrameEventDecodeBegin = 14
    EStreamFrameEventDecodeEnd = 15
    EStreamFrameEventUploadBegin = 16
    EStreamFrameEventUploadEnd = 17
    EStreamFrameEventComplete = 18


class EStreamFramerateLimiter(SteamIntEnum):
    EStreamFramerateSlowCapture = 1
    EStreamFramerateSlowConvert = 2
    EStreamFramerateSlowEncode = 4
    EStreamFramerateSlowNetwork = 8
    EStreamFramerateSlowDecode = 16
    EStreamFramerateSlowGame = 32
    EStreamFramerateSlowDisplay = 64


class EStreamFrameResult(SteamIntEnum):
    EStreamFrameResultPending = 0
    EStreamFrameResultDisplayed = 1
    EStreamFrameResultDroppedNetworkSlow = 2
    EStreamFrameResultDroppedNetworkLost = 3
    EStreamFrameResultDroppedDecodeSlow = 4
    EStreamFrameResultDroppedDecodeCorrupt = 5
    EStreamFrameResultDroppedLate = 6
    EStreamFrameResultDroppedReset = 7


class EStreamHostPlayAudioPreference(SteamIntEnum):
    EStreamHostPlayAudioDefault = 0
    EStreamHostPlayAudioAlways = 1


class EStreamingDataType(SteamIntEnum):
    EStreamingAudioData = 0
    EStreamingVideoData = 1
    EStreamingMicrophoneData = 2


class EStreamMouseButton(SteamIntEnum):
    EStreamMouseButtonLeft = 1
    EStreamMouseButtonRight = 2
    EStreamMouseButtonMiddle = 16
    EStreamMouseButtonX1 = 32
    EStreamMouseButtonX2 = 64
    EStreamMouseButtonUnknown = 4096


class EStreamMouseWheelDirection(SteamIntEnum):
    EStreamMouseWheelUp = 120
    EStreamMouseWheelDown = -120
    EStreamMouseWheelLeft = 3
    EStreamMouseWheelRight = 4


class EStreamP2PScope(SteamIntEnum):
    EStreamP2PScopeAutomatic = 0
    EStreamP2PScopeDisabled = 1
    EStreamP2PScopeOnlyMe = 2
    EStreamP2PScopeFriends = 3
    EStreamP2PScopeEveryone = 4


class EStreamQualityPreference(SteamIntEnum):
    EStreamQualityAutomatic = -1
    EStreamQualityFast = 1
    EStreamQualityBalanced = 2
    EStreamQualityBeautiful = 3


class EStreamStatsMessage(SteamIntEnum):
    EStreamStatsFrameEvents = 1
    EStreamStatsDebugDump = 2
    EStreamStatsLogMessage = 3
    EStreamStatsLogUploadBegin = 4
    EStreamStatsLogUploadData = 5
    EStreamStatsLogUploadComplete = 6


class EStreamVersion(SteamIntEnum):
    EStreamVersionNone = 0
    EStreamVersionCurrent = 1


class EStreamVideoCodec(SteamIntEnum):
    EStreamVideoCodecNone = 0
    EStreamVideoCodecRaw = 1
    EStreamVideoCodecVP8 = 2
    EStreamVideoCodecVP9 = 3
    EStreamVideoCodecH264 = 4
    EStreamVideoCodecHEVC = 5
    EStreamVideoCodecORBX1 = 6
    EStreamVideoCodecORBX2 = 7


class ESystemAudioChannel(SteamIntEnum):
    SystemAudioChannel_Invalid = 0
    SystemAudioChannel_Aggregated = 1
    SystemAudioChannel_FrontLeft = 2
    SystemAudioChannel_FrontRight = 3
    SystemAudioChannel_LFE = 4
    SystemAudioChannel_BackLeft = 5
    SystemAudioChannel_BackRight = 6
    SystemAudioChannel_FrontCenter = 7
    SystemAudioChannel_Unknown = 8
    SystemAudioChannel_Mono = 9


class ESystemAudioDirection(SteamIntEnum):
    SystemAudioDirection_Invalid = 0
    SystemAudioDirection_Input = 1
    SystemAudioDirection_Output = 2


class ESystemAudioPortDirection(SteamIntEnum):
    SystemAudioPortDirection_Invalid = 0
    SystemAudioPortDirection_Input = 1
    SystemAudioPortDirection_Output = 2


class ESystemAudioPortType(SteamIntEnum):
    SystemAudioPortType_Invalid = 0
    SystemAudioPortType_Unknown = 1
    SystemAudioPortType_Audio32f = 2
    SystemAudioPortType_Midi8b = 3
    SystemAudioPortType_Video32RGBA = 4


class ESystemFanControlMode(SteamIntEnum):
    SystemFanControlMode_Invalid = 0
    SystemFanControlMode_Disabled = 1
    SystemFanControlMode_Default = 2


class ESystemServiceState(SteamIntEnum):
    Unavailable = 0
    Disabled = 1
    Enabled = 2


class ETextFilterSetting(SteamIntEnum):
    ETextFilterSettingSteamLabOptedOut = 0
    ETextFilterSettingEnabled = 1
    ETextFilterSettingEnabledAllowProfanity = 2
    ETextFilterSettingDisabled = 3


class ETouchGesture(SteamIntEnum):
    ETouchGestureNone = 0
    ETouchGestureTouch = 1
    ETouchGestureTap = 2
    ETouchGestureDoubleTap = 3
    ETouchGestureShortPress = 4
    ETouchGestureLongPress = 5
    ETouchGestureLongTap = 6
    ETouchGestureTwoFingerTap = 7
    ETouchGestureTapCancelled = 8
    ETouchGesturePinchBegin = 9
    ETouchGesturePinchUpdate = 10
    ETouchGesturePinchEnd = 11
    ETouchGestureFlingStart = 12
    ETouchGestureFlingCancelled = 13


class EUpdaterState(SteamIntEnum):
    Invalid = 0
    UpToDate = 2
    Checking = 3
    Available = 4
    Applying = 5
    ClientRestartPending = 6
    SystemRestartPending = 7


class EUpdaterType(SteamIntEnum):
    Invalid = 0
    Client = 1
    OS = 2
    BIOS = 3
    Aggregated = 4
    Test1 = 5
    Test2 = 6
    Dummy = 7


EUserReviewScore = SteamIntEnum('EUserReviewScore', {
    'None': 0,
    'OverwhelminglyNegative': 1,
    'VeryNegative': 2,
    'Negative': 3,
    'MostlyNegative': 4,
    'Mixed': 5,
    'MostlyPositive': 6,
    'Positive': 7,
    'VeryPositive': 8,
    'OverwhelminglyPositive': 9,
})


class EUserReviewScorePreference(SteamIntEnum):
    Unset = 0
    IncludeAll = 1
    ExcludeBombs = 2


class EVideoFormat(SteamIntEnum):
    EVideoFormatNone = 0
    EVideoFormatYV12 = 1
    EVideoFormatAccel = 2


__all__ = [
    'E_STAR_GlyphWriteResult',
    'EACState',
    'EAgreementType',
    'EAudioFormat',
    'EAuthSessionGuardType',
    'EAuthSessionSecurityHistory',
    'EAuthTokenPlatformType',
    'EAuthTokenRevokeAction',
    'EAuthTokenState',
    'EBatteryState',
    'EBluetoothDeviceType',
    'EBroadcastChatPermission',
    'EBroadcastImageType',
    'EBroadcastWatchLocation',
    'EChatRoomGroupRank',
    'EChatRoomJoinState',
    'EChatRoomMemberStateChange',
    'EChatRoomMessageReactionType',
    'EChatRoomNotificationLevel',
    'EChatRoomServerMessage',
    'ECloudStoragePersistState',
    'EColorProfile',
    'ECommunityItemClass',
    'EContentCheckProvider',
    'ECPUGovernor',
    'EExternalAccountType',
    'EFrameAccumulatedStat',
    'EGameSearchAction',
    'EGameSearchResult',
    'EGetChannelsAlgorithm',
    'EGetGamesAlgorithm',
    'EGPUPerformanceLevel',
    'EGraphicsPerfOverlayLevel',
    'EInternalAccountType',
    'EKeyEscrowUsage',
    'ELobbyStatus',
    'ELogFileType',
    'EMarketingMessageAssociationType',
    'EMarketingMessageLookupType',
    'EMarketingMessageType',
    'EMarketingMessageVisibility',
    'EMessageReactionType',
    'ENotificationSetting',
    'EOSBranch',
    'EPlaytestStatus',
    'EProfileCustomizationStyle',
    'EProfileCustomizationType',
    'EProtoExecutionSite',
    'EProvideDeckFeedbackPreference',
    'EPublishedFileForSaleStatus',
    'EPublishedFileRevision',
    'EPublishedFileStorageSystem',
    'EScalingFilter',
    'ESDCardFormatStage',
    'ESessionPersistence',
    'ESteamDeckCompatibilityCategory',
    'ESteamDeckCompatibilityFeedback',
    'ESteamDeckCompatibilityResultDisplayType',
    'ESteamTVContentTemplate',
    'EStorageBlockContentType',
    'EStorageBlockFileSystemType',
    'EStoreAppType',
    'EStoreCategoryType',
    'EStoreDiscoveryQueueType',
    'EStoreItemType',
    'EStreamActivity',
    'EStreamAudioCodec',
    'EStreamBitrate',
    'EStreamChannel',
    'EStreamColorspace',
    'EStreamControllerConfigMsg',
    'EStreamControlMessage',
    'EStreamDataMessage',
    'EStreamDiscoveryMessage',
    'EStreamFrameEvent',
    'EStreamFramerateLimiter',
    'EStreamFrameResult',
    'EStreamHostPlayAudioPreference',
    'EStreamingDataType',
    'EStreamMouseButton',
    'EStreamMouseWheelDirection',
    'EStreamP2PScope',
    'EStreamQualityPreference',
    'EStreamStatsMessage',
    'EStreamVersion',
    'EStreamVideoCodec',
    'ESystemAudioChannel',
    'ESystemAudioDirection',
    'ESystemAudioPortDirection',
    'ESystemAudioPortType',
    'ESystemFanControlMode',
    'ESystemServiceState',
    'ETextFilterSetting',
    'ETouchGesture',
    'EUpdaterState',
    'EUpdaterType',
    'EUserReviewScore',
    'EUserReviewScorePreference',
    'EVideoFormat',
]



import logging

from eventemitter import EventEmitter
from steam.client.user import SteamUser
from steam.core.msg import MsgProto
from steam.enums import EResult, EFriendRelationship
from steam.enums.emsg import EMsg
from steam.steamid import SteamID, intBase


class Friends(object):
    def __init__(self, *args, **kwargs):
        super(Friends, self).__init__(*args, **kwargs)

        #: :class:`.SteamFriendlist` instance
        self.friends = SteamFriendlist(self, logger_name="%s.friends" % self.__class__.__name__)


class SteamFriendlist(EventEmitter):
    """SteamFriendlist is an object that keeps state of user's friend list.
    It's essentially a :class:`list` of :class:`.SteamUser`.
    You can iterate over it, check if it contains a particular ``steam id``,
    or get :class:`.SteamUser` for a ``steam id``.
    """
    EVENT_READY = 'ready'
    """Friend list is ready for use
    """
    EVENT_FRIEND_INVITE = 'friend_invite'
    """New or existing friend invite

    :param user: steam user instance
    :type user: :class:`.SteamUser`
    """
    EVENT_FRIEND_NEW = 'friend_new'
    """Friendship established (after being accepted, or accepting)

    :param user: steam user instance
    :type user: :class:`.SteamUser`
    """
    EVENT_FRIEND_REMOVED = 'friend_removed'
    """No longer a friend (removed by either side)

    :param user: steam user instance
    :type user: :class:`.SteamUser`
    """
    EVENT_FRIEND_ADD_RESULT = 'friend_add_result'
    """Result response after adding a friend

    :param eresult: result
    :type eresult: :class:`.EResult`
    :param steam_id: steam id
    :type steam_id: :class:`.SteamID`
    """

    ready = False  #: indicates whether friend list is ready for use

    def __init__(self, client, logger_name='SteamFriendList'):
        self._LOG = logging.getLogger(logger_name)
        self._fr = {}
        self._steam = client

        self._steam.on(EMsg.ClientAddFriendResponse, self._handle_add_friend_result)
        self._steam.on(EMsg.ClientFriendsList, self._handle_friends_list)
        self._steam.on(self._steam.EVENT_DISCONNECTED, self._handle_disconnect)

    def emit(self, event, *args):
        if event is not None:
            self._LOG.debug("Emit event: %s" % repr(event))
        EventEmitter.emit(self, event, *args)

    def _handle_disconnect(self):
        self.ready = False
        self._fr.clear()

    def _handle_add_friend_result(self, message):
        eresult = EResult(message.body.eresult)
        steam_id = SteamID(message.body.steam_id_added)
        self.emit(self.EVENT_FRIEND_ADD_RESULT, eresult, steam_id)

    def _handle_friends_list(self, message):
        incremental = message.body.bincremental
        if incremental == False:
            self._fr.clear()

        steamids_to_check = set()

        # update internal friends list
        for friend in message.body.friends:
            steamid = SteamID(friend.ulfriendid)

            if steamid.type != steamid.EType.Individual:
                continue

            suser = self._steam.get_user(steamid, False)
            rel = EFriendRelationship(friend.efriendrelationship)

            if steamid not in self._fr and rel != EFriendRelationship.NONE:  # 0
                self._fr[steamid] = suser
                suser.relationship = rel
                steamids_to_check.add(steamid)

                if rel in (2, 4):  # RequestRecipient = 2, RequestInitiator = 4
                    if rel == EFriendRelationship.RequestRecipient:
                        self.emit(self.EVENT_FRIEND_INVITE, suser)
            else:
                oldrel, suser.relationship = suser.relationship, rel

                if rel == EFriendRelationship.NONE:
                    suser = self._fr.pop(steamid, None)
                    if suser and oldrel not in (EFriendRelationship.Ignored, 0):
                        self.emit(self.EVENT_FRIEND_REMOVED, suser)
                elif oldrel in (2, 4) and rel == EFriendRelationship.Friend:
                    self.emit(self.EVENT_FRIEND_NEW, suser)

        # request persona state for any new entries
        if steamids_to_check:
            self._steam.request_persona_state(steamids_to_check)

        if not self.ready:
            self.ready = True
            self.emit(self.EVENT_READY)

    def __repr__(self):
        return "<%s %d users>" % (
            self.__class__.__name__,
            len(self._fr),
        )

    def __len__(self):
        return len(self._fr)

    def __iter__(self):
        return self._fr.values()

    def __list__(self):
        return list(iter(self))

    def __getitem__(self, key):
        if isinstance(key, SteamUser):
            key = key.steam_id
        return self._fr[key]

    def __contains__(self, friend):
        if isinstance(friend, SteamUser):
            friend = friend.steam_id
        return friend in self._fr

    def add(self, steamid_or_accountname_or_email):
        """
        Add/Accept a steam user to be your friend.
        When someone sends you an invite, use this method to accept it.

        :param steamid_or_accountname_or_email: steamid, account name, or email
        :type steamid_or_accountname_or_email: :class:`int`, :class:`.SteamID`, :class:`.SteamUser`, :class:`str`

        .. note::
            Adding by email doesn't not work. It's only mentioned for the sake of completeness.
        """
        m = MsgProto(EMsg.ClientAddFriend)

        if isinstance(steamid_or_accountname_or_email, (intBase, int)):
            m.body.steamid_to_add = steamid_or_accountname_or_email
        elif isinstance(steamid_or_accountname_or_email, SteamUser):
            m.body.steamid_to_add = steamid_or_accountname_or_email.steam_id
        else:
            m.body.accountname_or_email_to_add = steamid_or_accountname_or_email

        self._steam.send_job(m)

    def remove(self, steamid):
        """
        Remove a friend

        :param steamid: their steamid
        :type steamid: :class:`int`, :class:`.SteamID`, :class:`.SteamUser`
        """
        if isinstance(steamid, SteamUser):
            steamid = steamid.steam_id

        self._steam.send(MsgProto(EMsg.ClientRemoveFriend), {'friendid': steamid})

    def block(self, steamid):
        """
        Block Steam user

        :param steamid: their steamid
        :type  steamid: :class:`int`, :class:`.SteamID`, :class:`.SteamUser`
        :return: result
        :rtype: :class:`EResult`
        """
        if isinstance(steamid, SteamUser):
            steamid = steamid.steam_id
        elif not isinstance(steamid, SteamID):
            steamid = SteamID(steamid)

        resp = self._steam.send_um_and_wait("Player.IgnoreFriend#1",
                                            {"steamid": steamid},
                                            timeout=10)

        if not resp:
            return EResult.Timeout
        elif resp.header.eresult == EResult.OK:
            if steamid not in self._fr:
                self._fr[steamid] = self._steam.get_user(steamid, False)
            self._fr[steamid].relationship = EFriendRelationship(resp.body.friend_relationship)

        return resp.header.eresult

    def unblock(self, steamid):
        """
        Unblock Steam user

        :param steamid: their steamid
        :type  steamid: :class:`int`, :class:`.SteamID`, :class:`.SteamUser`
        :return: result
        :rtype: :class:`EResult`
        """
        if isinstance(steamid, SteamUser):
            steamid = steamid.steam_id
        elif not isinstance(steamid, SteamID):
            steamid = SteamID(steamid)

        resp = self._steam.send_um_and_wait("Player.IgnoreFriend#1",
                                            {"steamid": steamid, "unignore": True},
                                            timeout=10)

        if not resp:
            return EResult.Timeout
        elif resp.header.eresult == EResult.OK:
            if steamid in self._fr:
                self._fr[steamid].relationship = EFriendRelationship(resp.body.friend_relationship)

        return resp.header.eresult



"""
:class:`GameCoordinator` is used to proxy messages from/to GC.
It takes care of the encapsulation details, but on its own is not
enough to communicate with a given GC.

Example implementation of Dota 2 GC client with inheritance.

.. code:: python

    import myDotaModule
    from steam.client import SteamClient
    from steam.core.msg import GCMsgHdrProto
    from steam.client.gc import GameCoordinator

    class ExampleDotaClient(GameCoordinator):
        def __init__(self, steam):
            GameCoordinator.__init__(self, steam, 570)

        def _process_gc_message(self, emsg, header, body):
            if emsg == 4004: # EMsgGCClientWelcome
                message = myDotaModule.gcsdk_gcmessages_pb2.CMsgClientWelcome()
                message.ParseFromString(body)
                print message

        def send_hello(self):
            header = GCMsgHdrProto(4006)  # EMsgGCClientHello
            body = myDotaModule.gcsdk_gcmessages_pb2.CMsgClientHello()
            self.send(header, body.SerializeToString())

    client = SteamClient()
    dota = ExampleDotaClient(client)

    client.login()
    client.games_played([570])
    dota.send_hello()

The above code assumes that we have a ``myDotaModule`` that contains the appropriate
protobufs needed to (de)serialize message for communication with GC.
"""
import logging

from eventemitter import EventEmitter
from steam.client import SteamClient
from steam.core.msg import GCMsgHdr, GCMsgHdrProto, MsgProto
from steam.enums.emsg import EMsg
from steam.utils.proto import set_proto_bit, clear_proto_bit, is_proto


class GameCoordinator(EventEmitter):
    """
    ``GameCoordinator`` is used to proxy messages from/to GC

    :param steam_client: steam client instance
    :type steam_client: :class:`steam.client.SteamClient`
    :param app_id: app id of the application
    :type app_id: :class:`int`

    Incoming messages are emitted as events using their ``EMsg`` as an event identifier.

    :param header: message header
    :type header: :class:`steam.core.msg.GCMsgHdr`
    :param body: raw message body
    :type body: :class:`bytes`
    """

    def __init__(self, steam_client, app_id):
        if not isinstance(steam_client, SteamClient):
            raise ValueError("Expected an instance of SteamClient as first argument")

        self.steam = steam_client
        self.app_id = app_id
        self._LOG = logging.getLogger("GC(appid:%d)" % app_id)

        # listen for GC messages
        self.steam.on(EMsg.ClientFromGC, self._handle_from_gc)

    def emit(self, event, *args):
        if event is not None:
            self._LOG.debug("Emit event: %s" % repr(event))
        EventEmitter.emit(self, event, *args)

    def send(self, header, body):
        """
        Send a message to GC

        :param header: message header
        :type header: :class:`steam.core.msg.GCMsgHdr`
        :param body: serialized body of the message
        :type body: :class:`bytes`
        """
        message = MsgProto(EMsg.ClientToGC)
        message.header.routing_appid = self.app_id
        message.body.appid = self.app_id
        message.body.msgtype = (set_proto_bit(header.msg)
                                if header.proto
                                else header.msg
                                )
        message.body.payload = header.serialize() + body
        self.steam.send(message)

    def _handle_from_gc(self, msg):
        if msg.body.appid != self.app_id:
            return

        emsg = msg.body.msgtype

        if is_proto(emsg):
            header = GCMsgHdrProto(emsg, msg.body.payload)
            header_size = GCMsgHdrProto._size + header.headerLength
        else:
            header = GCMsgHdr(emsg, msg.body.payload)
            header_size = GCMsgHdr._size

        body = msg.body.payload[header_size:]

        self._process_gc_message(clear_proto_bit(emsg), header, body)

    def _process_gc_message(self, emsg, header, body):
        self.emit(emsg, header, body)



# -*- coding: utf-8 -*-
"""
This module simplifies the process of obtaining an authenticated session for steam websites.
After authentication is completed, a :class:`requests.Session` is created containing the auth cookies.
The session can be used to access ``steamcommunity.com``, ``store.steampowered.com``, and ``help.steampowered.com``.

.. warning::
    A web session may expire randomly, or when you login from different IP address.
    Some pages will return status code `401` when that happens.
    Keep in mind if you are trying to write robust code.

.. note::
    If you are using :class:`.SteamClient` take a look at :meth:`.SteamClient.get_web_session()`

.. note::
    If you need to authenticate as a mobile device for things like trading confirmations
    use :class:`MobileWebAuth` instead. The login process is identical, and in addition
    you will get :attr:`.oauth_token`.


Example usage:

.. code:: python

    import steam.webauth as wa

    user = wa.WebAuth('username')

    # At a console, cli_login can be used to easily perform all login steps
    session = user.cli_login('password')
    session.get('https://store.steampowered.com/account/history')

    # Or the login steps be implemented for other situation like so
    try:
        user.login('password')
    except (wa.CaptchaRequired, wa.LoginIncorrect) as exp:
        if isinstance(exp, LoginIncorrect):
            # ask for new password
        else:
            password = self.password

        if isinstance(exp, wa.CaptchaRequired):
            print user.captcha_url
            # ask a human to solve captcha
        else:
            captcha = None

        user.login(password=password, captcha=captcha)
    except wa.EmailCodeRequired:
        user.login(email_code='ZXC123')
    except wa.TwoFactorCodeRequired:
        user.login(twofactor_code='ZXC123')

    user.session.get('https://store.steampowered.com/account/history/')

"""
import json
from base64 import b64encode
from getpass import getpass
from time import time

import requests
from steam.core.crypto import rsa_publickey, pkcs1v15_encrypt
from steam.enums.proto import EAuthSessionGuardType
from steam.steamid import SteamID
from steam.utils.web import generate_session_id

intBase = int
_cli_input = input

API_HEADERS = {
    'origin': 'https://steamcommunity.com',
    'referer': 'https://steamcommunity.com/',
    'accept': 'application/json, text/plain, */*'
}

API_URL = 'https://api.steampowered.com/{}Service/{}/v{}'

SUPPORTED_AUTH_TYPES = [EAuthSessionGuardType.EmailCode, EAuthSessionGuardType.DeviceCode,
                        EAuthSessionGuardType.DeviceConfirmation]


class WebAuth(object):
    """New WEB Auth class.

    This class works with Steam API:
        https://steamapi.xpaw.me/#IAuthenticationService

    Currently, supports bsaic login/password auth with no 2FA, 2FA via
    steam guard code and 2FA via EMAIL confirmation.

    TODO: Add QR code support.

    TODO: Fully rework api handling. PUT api into separate class,
      in order to make this class responsible only for actual auth.

    IMPORTANT:
        Actually, at real login page
        steam handles function little bit different.
        e.g. https://api.steampowered.com/IAuthenticationService/BeginAuthSessionViaCredentials/v1
        can handle multipart/form-data; with something like.

        {
            input_protobuf_encoded: EgxhbmFjb25kYXJ0dXIa2AJodHgzZzlJaWY4dGE4RGxqR2VWbncwZWpxdi9uNDByRkZxaFduVk12VFFhRm1ZU1F4MHlrYUlJNmFURlVJWEQ5Z2VXMTlObWJDc3pydmNQZ1RTVllyOWl0SW5EWjgzMVh0YWtOaHJaUk9JN1lvMzhpb2xHRmdHdVBZT3NsekErTHZNZlJoQ3YzL1JFaEpNQlhjaXhzNklRRTZjbnM4d1JWbGI0TVA1Nzd0MHpGajRpcWF4U21KbnVjRDh5YzVIVkYvMERlMnFKd3dGTG0vR3B4SEdreFFlQURzZi9OTXJMTUszcWxnR3NLZm4ycGxNOGhkMzF3YnErSUlCZkNJb3dFZWExaUpJcmVjYkdLT0EvRlJ5VFpSQlVoVitLQmt6TGk3THY1UjVNYVRJSzNPTCtCMUZnZ2xWSG94c0ErTm5BMHVqSVZWZ0ZRdGpDL2tMTjd0SmhYamc9PSCA+aCT0ggoATgBQgVTdG9yZUqBAQp9TW96aWxsYS81LjAgKFdpbmRvd3MgTlQgMTAuMDsgV2luNjQ7IHg2NCkgQXBwbGVXZWJLaXQvNTM3LjM2IChLSFRNTCwgbGlrZSBHZWNrbykgQ2hyb21lLzExNi4wLjAuMCBTYWZhcmkvNTM3LjM2IE9QUi8xMDIuMC4wLjAQAlgI
        }
        it's protobuf encoded value. You can decode it here:
            https://protobuf-decoder.netlify.app

        some fields I can understand:
            2) string - steamlogin
            3) string - encrypted password
            4) timestamp to map to a key - STime
            5) some INT (probably it's always 1) it's DEPRECATED
            7) whether we are requesting a persistent or an ephemeral session
            8) (EMachineAuthWebDomain) identifier of client requesting auth.
               e.g. "Store"
            9) Protobuf of device type (see CAuthentication_DeviceDetails):
                9.1) string - User-Agent
                9.2) Int - platform identifier e.g. 2 (means Web Browser)
                    (See EAuthTokenPlatformType protobuf).
                9.3) os_type (MOSTLY NOT PRESENTED IN REAL REQUESTS)
                9.4) gaming_device_type (MOSTLY NOT PRESENTED IN REAL REQUESTS)
            11) UNDOCUMENTED AT ALL: Some number (like 8)

            FIELD NUMBERS I SKIPPED MEANS THEY ARE NOT PRESENTED IN REAL REQUEST
        We currently uses basic multipart/form-data and "key-value"
        data presentation.
        But I  Think, it's important to know, that real steam works differently,
        and maybe we can once upon a time simulate it's REAL behavior.

    """

    # Pretend to be chrome on windows, made this act as most like a
    # browser as possible to (hopefully) avoid breakage in the future from valve
    def __init__(self, username='', password='',
                 userAgent='Mozilla/5.0 (Windows NT 10.0; Win64; x64)'
                           ' AppleWebKit/537.36 (KHTML, like Gecko)'
                           ' Chrome/118.0.0.0 Safari/537.36'):

        # ALL FUNCTIONS RENAMED TO PEP8 NOTATION.
        self.session = requests.session()
        self.user_agent = userAgent
        self.username = username
        self.password = password
        self.session.headers['User-Agent'] = self.user_agent
        self.steam_id = None
        self.client_id = None
        self.request_id = None
        self.refresh_token = None
        self.access_token = None
        self.session_id = None
        self.email_auth_waits = False  # Not used yet.
        self.logged_on = False

    @staticmethod
    def send_api_request(data, steam_api_interface, steam_api_method,
                         steam_api_version):
        """Send request to Steam API via requests"""
        steam_url = API_URL.format(steam_api_interface, steam_api_method,
                                   steam_api_version)

        if steam_api_method == "GetPasswordRSAPublicKey":  # It's GET method
            res = requests.get(steam_url, timeout=10, headers=API_HEADERS,
                               params=data)
        else:  # Every other API endpoints are POST.
            res = requests.post(steam_url, timeout=10, headers=API_HEADERS,
                                data=data)

        res.raise_for_status()
        return res.json()

    def _get_rsa_key(self):
        """Get rsa key to crypt password."""
        return self.send_api_request({'account_name': self.username},
                                     "IAuthentication", 'GetPasswordRSAPublicKey', 1)

    def _encrypt_password(self):
        """Encrypt password via RSA key

        Steam handles every password only in encoded way.
        """
        r = self._get_rsa_key()

        mod = intBase(r['response']['publickey_mod'], 16)
        exp = intBase(r['response']['publickey_exp'], 16)

        pub_key = rsa_publickey(mod, exp)
        encrypted = pkcs1v15_encrypt(pub_key, self.password.encode('ascii'))
        b64 = b64encode(encrypted)

        return tuple((b64.decode('ascii'), r['response']['timestamp']))

    def _startSessionWithCredentials(self, account_encrypted_password: str,
                                     time_stamp: int):
        """Start login session via BeginAuthSessionViaCredentials


        """
        resp = self.send_api_request(
            {'device_friendly_name': self.user_agent,
             'account_name': self.username,
             'encrypted_password': account_encrypted_password,
             'encryption_timestamp': time_stamp,
             'remember_login': '1',
             'platform_type': '2',
             'persistence': '1',
             'website_id': 'Community',
             },
            'IAuthentication',
            'BeginAuthSessionViaCredentials',
            1
        )
        self.client_id = resp['response']['client_id']
        self.request_id = resp['response']['request_id']
        self.steam_id = SteamID(resp['response']['steamid'])
        self.allowed_confirmations = [EAuthSessionGuardType(confirm_type['confirmation_type']) for confirm_type in
                                      resp['response']['allowed_confirmations']]

    def _startLoginSession(self):
        """Starts login session via credentials."""
        encrypted_password = self._encrypt_password()
        self._startSessionWithCredentials(encrypted_password[0],
                                          encrypted_password[1])

    def _pollLoginStatus(self):
        """Get status of current Login Session

        This function asks server about login session status.
        If we logged in, this returns access_token that we needed.

        TODO: add check of interval, returned from _startSessionWithCredentials
            actually it has no need now, but
        """
        resp = self.send_api_request({
            'client_id': str(self.client_id),
            'request_id': str(self.request_id)
        }, 'IAuthentication', 'PollAuthSessionStatus', 1)
        try:
            self.refresh_token = resp['response']['refresh_token']
            self.access_token = resp['response']['access_token']
        except KeyError:
            raise TwoFactorAuthNotProvided('Authentication requires 2fa token, which is not provided or invalid')

    def _finalizeLogin(self):
        self.sessionID = generate_session_id()
        self.logged_on = True
        for domain in ['store.steampowered.com', 'help.steampowered.com',
                       'steamcommunity.com']:
            self.session.cookies.set('sessionid', self.sessionID, domain=domain)
            self.session.cookies.set('steamLoginSecure',
                                     str(self.steam_id.as_64) + "||" + str(
                                         self.access_token), domain=domain)

    def _update_login_token(
            self,
            code: str,
            code_type: EAuthSessionGuardType = EAuthSessionGuardType.DeviceCode
    ):
        """Send 2FA token to steam

        Please note, that very rare, login can be unsuccessful,
        if you use code, that guard provided you BEFORE log in session
        was started. To fix it, just rerun login.

        """
        data = {
            'client_id': self.client_id,
            'steamid': self.steam_id,
            'code': code,
            'code_type': code_type
        }
        res = self.send_api_request(data, 'IAuthentication',
                                    'UpdateAuthSessionWithSteamGuardCode', 1)
        return res

    def login(self, username: str = '', password: str = '', code: str = None,
              email_required=False):
        """Log in user by new Steam API

        If user has no need 2FA, this function will just  log in user.
        If 2FA SteamGuard code needed, when user can provide it just
        with guard.SteamAuthenticator.get_code like it always was.

        If Email code is required, when user can provide email_required.
        If email_required was provided, when this function only setup auth
        and return new function.

        this function will receive email code. Once email code will be provided
        authentication process will be complete.
        If wrong code provided in this new function, when error will be raised.
        And new code will be waited.

        """
        if self.logged_on:
            return self.session

        username = username or self.username
        password = password or self.password

        if username == '' or password == '':
            raise LoginIncorrect("Username or password is provided empty!")
        else:
            self.username = username
            self.password = password

        self._startLoginSession()
        if code:
            self._update_login_token(code)
        if email_required:
            # We do another request, which force steam to send email code
            # (otherwise code just not sent).

            url = (f'https://login.steampowered.com/jwt/checkdevice/'
                   f'{self.steam_id}')
            res = self.session.post(url, data={
                'clientid': self.client_id,
                'steamid': self.steam_id
            }).json()
            if res.get('result') == 8:
                # This usually mean code sent now.
                def end_login(email_code: str):
                    self._update_login_token(email_code,
                                             EAuthSessionGuardType.EmailCode)
                    self._pollLoginStatus()
                    self._finalizeLogin()
                    return self.session

                return end_login
            if res.get('result') == 29:
                # This code 100% means some data not valid
                # Actually this must will never be called, because
                # Errors can be only like wrong cookies. (Theoretically)
                raise WebAuthException("Something invalid went. Try again later.")
        self._pollLoginStatus()
        self._finalizeLogin()

        return self.session

    def logout_everywhere(self):
        """Log out on every device.

        This function works just like button at
        https://store.steampowered.com/twofactor/manage
        and allows user to logout on every device.
        Can be VERY useful e.g. for users, who practice account rent.
        """
        session_id = self.session.cookies.get('sessionid',
                                              domain='store.steampowered.com')

        # By the times I saw session can be both of keys, so select valid.
        session_id = session_id or self.session.cookies.get(
            'sessionId', domain='store.steampowered.com')
        data = {
            "action": "deauthorize",
            "sessionid": session_id
        }
        resp = self.session.post(
            'https://store.steampowered.com/twofactor/manage_action',
            data=data
        )

        return resp.status_code == 200

    def cli_login(self, username: str = '', password: str = '', code: str = '',
                  email_required: bool = False):
        """Generates CLI prompts to perform the entire login process

        If you use email confirm, provide email_required = True,
        else just provide code.
        """
        while True:
            try:
                res = self.login(
                    username,
                    password,
                    code,
                    email_required
                )
            except LoginIncorrect:
                prompt = ("Enter password for %s: " if not password else "Invalid password for %s. Enter password:")
                password = getpass(prompt % repr(self.username))
                continue
            except TwoFactorAuthNotProvided:
                # 2FA handling
                allowed = set(self.allowed_confirmations)
                if allowed.isdisjoint(SUPPORTED_AUTH_TYPES):
                    raise AuthTypeNotSupported("Couldn't find a supported auth type for this account.")

                can_confirm_with_app = EAuthSessionGuardType.DeviceConfirmation in allowed

                twofactor_code = ''
                while twofactor_code.strip() == '':
                    prompt = ''
                    if EAuthSessionGuardType.DeviceCode in allowed:
                        prompt = 'Enter your Steam Guard code'
                    elif EAuthSessionGuardType.EmailCode in allowed:
                        prompt = 'Enter the Steam 2FA code emailed to you'
                    if can_confirm_with_app:
                        if prompt == '':
                            # don't think this is possible, but let's handle it anyways
                            input("Please confirm Steam Guard on your device and press ENTER to continue")
                        else:
                            prompt += ' (or simply press Enter if approved via app)'

                    if prompt != '':
                        prompt += ': '
                        twofactor_code = input(prompt)

                    if can_confirm_with_app:
                        try:
                            self._pollLoginStatus()  # test to see if they've authenticated via app
                            break
                        except TwoFactorAuthNotProvided:
                            # they've not authenticated via the app, let's see if we can use their provided code
                            pass

                    if twofactor_code.strip():
                        using_email_code = EAuthSessionGuardType.EmailCode in allowed
                        self._update_login_token(twofactor_code,
                                                 EAuthSessionGuardType.EmailCode if using_email_code else EAuthSessionGuardType.DeviceCode)
                        try:
                            self._pollLoginStatus()
                            break
                        except TwoFactorAuthNotProvided:
                            print("Invalid auth code. Please try again")
                            twofactor_code = ''
                            continue
                    else:
                        print("Unauthenticated. Please try again")
                self._finalizeLogin()
                return self.session
            if hasattr(res, '__call__'):
                # this should not ever happen at this point as we don't use `email_required` on login()
                while True:
                    try:
                        twofactor_code = input('Enter your 2fa/email code: ')
                        resp = res(twofactor_code)
                        return resp
                    except WebAuthException:
                        pass
            else:
                return self.session


# TODO: DEPRECATED, must be rewritten, like WebAuth
class MobileWebAuth(WebAuth):
    """Identical to :class:`WebAuth`, except it authenticates as a mobile device."""
    oauth_token = None  #: holds oauth_token after successful login

    def _send_login(self, password='', captcha='', email_code='',
                    twofactor_code=''):
        data = {
            'username': self.username,
            "password": b64encode(
                pkcs1v15_encrypt(self.key, password.encode('ascii'))),
            "emailauth": email_code,
            "emailsteamid": str(self.steam_id) if email_code else '',
            "twofactorcode": twofactor_code,
            "captchagid": self.captcha_gid,
            "captcha_text": captcha,
            "loginfriendlyname": "python-steam webauth",
            "rsatimestamp": self.timestamp,
            "remember_login": 'true',
            "donotcache": int(time() * 100000),
            "oauth_client_id": "DE45CD61",
            "oauth_scope": "read_profile write_profile read_client write_client",
        }

        self.session.cookies.set('mobileClientVersion', '0 (2.1.3)')
        self.session.cookies.set('mobileClient', 'android')

        try:
            return self.session.post(
                'https://steamcommunity.com/login/dologin/', data=data,
                timeout=15).json()
        except requests.exceptions.RequestException as e:
            raise HTTPError(str(e))
        finally:
            self.session.cookies.pop('mobileClientVersion', None)
            self.session.cookies.pop('mobileClient', None)

    def _finalize_login(self, login_response):
        data = json.loads(login_response['oauth'])
        self.steam_id = SteamID(data['steamid'])
        self.oauth_token = data['oauth_token']

    def oauth_login(self, oauth_token='', steam_id='', language='english'):
        """Attempts a mobile authenticator login using an oauth token, which can be obtained from a previously logged-in
        `MobileWebAuth`

        :param oauth_token: oauth token string, if it wasn't provided on instance init
        :type  oauth_token: :class:`str`
        :param steam_id: `SteamID` of the account to log into, if it wasn't provided on instance init
        :type  steam_id: :class:`str` or :class:`SteamID`
        :param language: select language for steam web pages (sets language cookie)
        :type  language: :class:`str`
        :return: a session on success and :class:`None` otherwise
        :rtype: :class:`requests.Session`, :class:`None`
        :raises HTTPError: any problem with http request, timeouts, 5xx, 4xx etc
        :raises LoginIncorrect: Invalid token or SteamID
        """
        if oauth_token:
            self.oauth_token = oauth_token
        elif self.oauth_token:
            oauth_token = self.oauth_token
        else:
            raise LoginIncorrect('token is not specified')

        if steam_id:
            self.steam_id = SteamID(steam_id)
        elif not self.steam_id:
            raise LoginIncorrect('steam_id is not specified')

        steam_id = self.steam_id.as_64

        data = {
            'access_token': oauth_token
        }

        try:
            resp = self.session.post(
                'https://api.steampowered.com/IMobileAuthService/GetWGToken/v0001',
                data=data)
        except requests.exceptions.RequestException as e:
            raise HTTPError(str(e))

        try:
            resp_data = resp.json()['response']
        except json.decoder.JSONDecodeError as e:
            if 'Please verify your <pre>key=</pre> parameter.' in resp.text:
                raise LoginIncorrect('invalid token')
            else:
                raise e

        self.session_id = generate_session_id()

        for domain in ['store.steampowered.com', 'help.steampowered.com',
                       'steamcommunity.com']:
            self.session.cookies.set('birthtime', '-3333', domain=domain)
            self.session.cookies.set('sessionid', self.session_id,
                                     domain=domain)
            self.session.cookies.set('mobileClientVersion', '0 (2.1.3)',
                                     domain=domain)
            self.session.cookies.set('mobileClient', 'android', domain=domain)
            self.session.cookies.set('steamLogin',
                                     str(steam_id) + "%7C%7C" + resp_data[
                                         'token'], domain=domain)
            self.session.cookies.set('steamLoginSecure',
                                     str(steam_id) + "%7C%7C" + resp_data[
                                         'token_secure'],
                                     domain=domain, secure=True)
            self.session.cookies.set('Steam_Language', language, domain=domain)

        self.logged_on = True

        return self.session


class WebAuthException(Exception):
    pass


class AuthTypeNotSupported(WebAuthException):
    pass


class TwoFactorAuthNotProvided(WebAuthException):
    pass


class HTTPError(WebAuthException):
    pass


class LoginIncorrect(WebAuthException):
    pass


class CaptchaRequired(WebAuthException):
    pass


class CaptchaRequiredLoginIncorrect(CaptchaRequired, LoginIncorrect):
    pass


class EmailCodeRequired(WebAuthException):
    pass


class TwoFactorCodeRequired(WebAuthException):
    pass


class TooManyLoginFailures(WebAuthException):
    pass



import re
from importlib import import_module

service_lookup = {  # MARK_SERVICE_START
    'Authentication': 'steam.protobufs.steammessages_auth_pb2',
    'AuthenticationSupport': 'steam.protobufs.steammessages_auth_pb2',
    'CloudGaming': 'steam.protobufs.steammessages_auth_pb2',
    'Broadcast': 'steam.protobufs.steammessages_broadcast_pb2',
    'BroadcastClient': 'steam.protobufs.steammessages_broadcast_pb2',
    'Chat': 'steam.protobufs.steammessages_chat_pb2',
    'ChatRoom': 'steam.protobufs.steammessages_chat_pb2',
    'ClanChatRooms': 'steam.protobufs.steammessages_chat_pb2',
    'ChatRoomClient': 'steam.protobufs.steammessages_chat_pb2',
    'ChatUsability': 'steam.protobufs.steammessages_chat_pb2',
    'ChatUsabilityClient': 'steam.protobufs.steammessages_chat_pb2',
    'Cloud': 'steam.protobufs.steammessages_cloud_pb2',
    'CloudClient': 'steam.protobufs.steammessages_cloud_pb2',
    'ContentServerDirectory': 'steam.protobufs.steammessages_contentsystem_pb2',
    'Credentials': 'steam.protobufs.steammessages_credentials_pb2',
    'DataPublisher': 'steam.protobufs.steammessages_datapublisher_pb2',
    'ValveHWSurvey': 'steam.protobufs.steammessages_datapublisher_pb2',
    'ContentBuilder': 'steam.protobufs.steammessages_depotbuilder_pb2',
    'DeviceAuth': 'steam.protobufs.steammessages_deviceauth_pb2',
    'Econ': 'steam.protobufs.steammessages_econ_pb2',
    'FriendMessages': 'steam.protobufs.steammessages_friendmessages_pb2',
    'FriendMessagesClient': 'steam.protobufs.steammessages_friendmessages_pb2',
    'GameNotifications': 'steam.protobufs.steammessages_gamenotifications_pb2',
    'GameNotificationsClient': 'steam.protobufs.steammessages_gamenotifications_pb2',
    'GameServers': 'steam.protobufs.steammessages_gameservers_pb2',
    'GameServerClient': 'steam.protobufs.steammessages_gameservers_pb2',
    'Inventory': 'steam.protobufs.steammessages_inventory_pb2',
    'InventoryClient': 'steam.protobufs.steammessages_inventory_pb2',
    'CommunityLinkFilter': 'steam.protobufs.steammessages_linkfilter_pb2',
    'LobbyMatchmakingLegacy': 'steam.protobufs.steammessages_lobbymatchmaking_pb2',
    'MarketingMessages': 'steam.protobufs.steammessages_marketingmessages_pb2',
    'EconMarket': 'steam.protobufs.steammessages_market_pb2',
    'Offline': 'steam.protobufs.steammessages_offline_pb2',
    'Parental': 'steam.protobufs.steammessages_parental_pb2',
    'ParentalClient': 'steam.protobufs.steammessages_parental_pb2',
    'Parties': 'steam.protobufs.steammessages_parties_pb2',
    'PartnerApps': 'steam.protobufs.steammessages_partnerapps_pb2',
    'PhysicalGoods': 'steam.protobufs.steammessages_physicalgoods_pb2',
    'Player': 'steam.protobufs.steammessages_player_pb2',
    'PlayerClient': 'steam.protobufs.steammessages_player_pb2',
    'PublishedFile': 'steam.protobufs.steammessages_publishedfile_pb2',
    'PublishedFileClient': 'steam.protobufs.steammessages_publishedfile_pb2',
    'QueuedMatchmaking': 'steam.protobufs.steammessages_qms_pb2',
    'QueuedMatchmakingGameHost': 'steam.protobufs.steammessages_qms_pb2',
    'Secrets': 'steam.protobufs.steammessages_secrets_pb2',
    'Shader': 'steam.protobufs.steammessages_shader_pb2',
    'SiteManagerClient': 'steam.protobufs.steammessages_site_license_pb2',
    'SiteLicense': 'steam.protobufs.steammessages_site_license_pb2',
    'STAR': 'steam.protobufs.steammessages_star_pb2',
    'SteamTV': 'steam.protobufs.steammessages_steamtv_pb2',
    'StoreBrowse': 'steam.protobufs.steammessages_storebrowse_pb2',
    'Store': 'steam.protobufs.steammessages_store_pb2',
    'StoreClient': 'steam.protobufs.steammessages_store_pb2',
    'TimedTrial': 'steam.protobufs.steammessages_timedtrial_pb2',
    'TwoFactor': 'steam.protobufs.steammessages_twofactor_pb2',
    'TestSteamClient': 'steam.protobufs.steammessages_unified_test_pb2',
    'TestServerFromClient': 'steam.protobufs.steammessages_unified_test_pb2',
    'UserAccount': 'steam.protobufs.steammessages_useraccount_pb2',
    'AccountLinking': 'steam.protobufs.steammessages_useraccount_pb2',
    'EmbeddedClient': 'steam.protobufs.steammessages_useraccount_pb2',
    'Video': 'steam.protobufs.steammessages_video_pb2',
    'VideoClient': 'steam.protobufs.steammessages_video_pb2',
    'FovasVideo': 'steam.protobufs.steammessages_video_pb2',
    'HelpRequestLogs': 'steam.protobufs.steammessages_webui_friends_pb2',
    'Community': 'steam.protobufs.steammessages_webui_friends_pb2',
    'ExperimentService': 'steam.protobufs.steammessages_webui_friends_pb2',
    'FriendsList': 'steam.protobufs.steammessages_webui_friends_pb2',
    'FriendsListClient': 'steam.protobufs.steammessages_webui_friends_pb2',
    'Clan': 'steam.protobufs.steammessages_webui_friends_pb2',
    'VoiceChat': 'steam.protobufs.steammessages_webui_friends_pb2',
    'VoiceChatClient': 'steam.protobufs.steammessages_webui_friends_pb2',
    'WebRTCClient': 'steam.protobufs.steammessages_webui_friends_pb2',
    'WebRTCClientNotifications': 'steam.protobufs.steammessages_webui_friends_pb2',
    'MobilePerAccount': 'steam.protobufs.steammessages_webui_friends_pb2',
    'MobileDevice': 'steam.protobufs.steammessages_webui_friends_pb2',
    'Workshop': 'steam.protobufs.steammessages_workshop_pb2',
}  # MARK_SERVICE_END

method_lookup = {}


def get_um(method_name, response=False):
    """Get protobuf for given method name

    :param method_name: full method name (e.g. ``Player.GetGameBadgeLevels#1``)
    :type method_name: :class:`str`
    :param response: whether to return proto for response or request
    :type response: :class:`bool`
    :return: protobuf message
    """
    key = (method_name, response)

    if key not in method_lookup:
        match = re.findall(r'^([a-z]+)\.([a-z]+)#(\d)?$', method_name, re.I)
        if not match:
            return None

        interface, method, version = match[0]

        if interface not in service_lookup:
            return None

        package = import_module(service_lookup[interface])

        service = getattr(package, interface, None)
        if service is None:
            return None

        for method_desc in service.GetDescriptor().methods:
            name = "%s.%s#%d" % (interface, method_desc.name, 1)

            method_lookup[(name, False)] = getattr(package, method_desc.input_type.full_name, None)
            method_lookup[(name, True)] = getattr(package, method_desc.output_type.full_name, None)

    return method_lookup[key]
